local WidgetLib = TidyPlatesWidgets
local LocalVars = TidyPlatesGreyTankVariables
local theme = TidyPlatesThemeList["Grey/Tank"]
local valueToString = TidyPlatesUtility.abbrevNumber

---------------
-- Text Delegates
---------------

local function SpellTextDelegate(unit)
	local spellname
	if unit.isCasting then 
		spellname = UnitCastingInfo("target") or UnitChannelInfo("target")
		return spellname
	else return "" end
end

local HealthTextFunctions = {
	--HEALTH_NONE
	function (health, healthmax) return "" end,
	--HEALTH_PCT
	function (health, healthmax) if health ~= healthmax then return "%"..ceil(100*(health/healthmax)) else return nil end end,
	--HEALTH_TOTAL
	function (health, healthmax) return valueToString(health) end,
	--HEALTH_DEF
	function (health, healthmax) if health ~= healthmax then return "-"..valueToString(healthmax - health) end end,
	--HEALTH_TOT_PCT
	function (health, healthmax) return valueToString(health).." / "..valueToString(healthmax).." ("..ceil(100*(health/healthmax)).."%)" end,
	--Health_Total_Pct_Def
	function (health, healthmax) return "+"..valueToString(health).." ("..ceil(100*(health/healthmax)).."%) -"..valueToString(healthmax - health) end,
}

local function HealthTextDelegate(unit)  
	return HealthTextFunctions[LocalVars.HealthText](unit.health, unit.healthmax)
end

---------------
-- Graphics Delegates
---------------
local function TankScale(unit)
	if InCombatLockdown() and unit.reaction == "HOSTILE" and  unit.threatSituation ~= "HIGH" then
		return LocalVars.ScaleLoose
	end
	return LocalVars.ScaleGeneral
end
	
local function TankAlpha(unit)
	if unit.isTarget or (not UnitExists("target")) then return 1
	else 	
		if LocalVars.OpacityHideNeutral and unit.reaction == "NEUTRAL" then return 0 end
		return LocalVars.OpacityNonTarget, true
	end
end

local function HealthColorDelegate(unit)	
	if  LocalVars.AggroHealth then
		if InCombatLockdown() and unit.reaction ~= "FRIENDLY" then
			local loose, tanking = LocalVars.AggroLooseColor, LocalVars.AggroTankedColor
			if unit.threatSituation ~= "HIGH" then return loose.r, loose.g, loose.b
				else return tanking.r, tanking.g, tanking.b end 
			end
	end
	return unit.red, unit.green, unit.blue
end

---------------
-- Widgets
---------------
local function OnInitializeDelegate(plate)
	if LocalVars.WidgetTug then 
		plate.WidgetTug = WidgetLib.CreateThreatLineWidget(plate)
		plate.WidgetTug:SetPoint("CENTER", plate, 0, 18)
	end
	--[[
	if LocalVars.WidgetTanked then
		plate.Tanked = WidgetLib.CreateTankedWidget(plate)
		plate.Tanked:SetPoint("CENTER", plate, 0, 17)
		plate.Tanked:SetFrameLevel(plate:GetFrameLevel()+2)
	end
	--]]
 end

local function OnUpdateDelegate(plate, unit)
	if LocalVars.WidgetTug then
		if not plate.WidgetTug then OnInitializeDelegate(plate) end
		plate.WidgetTug:UpdateThreat(unit)
	end
	--[[
	if LocalVars.WidgetTanked then
		if not plate.Tanked then OnInitializeDelegate(plate) end
		plate.Tanked:Update(unit)
	end	
	--]]
end

---------------
-- Function Assignment - Tank Mode
---------------
theme.SetSpecialText = HealthTextDelegate
theme.SetSpecialText2 = SpellTextDelegate
theme.SetScale = TankScale
theme.SetAlpha = TankAlpha
theme.OnUpdate = OnUpdateDelegate
theme.OnInitialize = OnInitializeDelegate
theme.SetHealthbarColor = HealthColorDelegate





